@@ -3,6 +3,8 @@
 /* ѹ                                                                   */
 /*----------------------------------------------------------------------------*/
 /* 1.0 : APIб 2003/06/20                                           */
+/* 1.1 : ХåȤοϰϤϲǽˤ 2004/08/09                        */
+/* 1.2 : ꡼ 2004/10/31                                          */
 /*============================================================================*/
 #include <musashi.h>
 #include <stdlib.h>
@@ -12,7 +14,7 @@
 #include <xtbucketHelp.h>
 struct mssComHelp comHelp={
   "xtbucket",     /* ޥ̾       */
-  "1.0",          /* С       */
+  "1.2",          /* С       */
   HELPT,          /* ޥɥȥ */
   HELPS,          /*              */
   HELPE,          /*            */
@@ -190,6 +192,15 @@ void calOptCutPoint(int cutNum){
   struct kTbl *tmpKtbl;
   int j,k,l;
 
+  /* cTableCnt==0 ȤȤϡͤNULL */
+  if(cTableCnt==0){
+    kTable=NULL;
+    sTable=NULL;
+    rTable=NULL;
+    cutCnt=0;
+    return;
+  }
+
   maxCnt=cTableCnt; /* ͤμ */
   /*kTableꥢ*/
   kTable = (struct kTbl *)mssCalloc(
@@ -406,6 +417,17 @@ int main(int argc, char *argv[]){
     INFC    /* ΥץΥ(Helpɽ)                         */
   };
 
+/*----------------------------------------------------------------------------*/
+/* ХåϰϽϥե                                                   */
+/*----------------------------------------------------------------------------*/
+  MssOptOTF optRNG={
+    OOTF,   /* ץ󥿥                                             */
+    "O",    /* (ʣʸԲ)                                   */
+    0,      /* 0:ץ, 1:ɬ                                         */
+    RNGT,   /* ΥץΥȥ(Helpɽ)                         */
+    RNGC    /* ΥץΥ(Helpɽ)                         */
+  };
+
 /*----------------------------------------------------------------------------*/
 /* ϥե                                                               */
 /*----------------------------------------------------------------------------*/
@@ -457,15 +479,17 @@ int main(int argc, char *argv[]){
 /* ץޤȤ                                                       */
 /*----------------------------------------------------------------------------*/
   void *opt[]={&optKEY,&optFLD,&optNUM,&optFMT,&optTyp,&optREV,
-               &optINF,&optOTF,&optZIP,&optTXT,&optTMP,NULL};
+               &optINF,&optRNG,&optOTF,&optZIP,&optTXT,&optTMP,NULL};
 
 /*============================================================================*/
 /* ѿ                                                             */
 /*============================================================================*/
   struct mssHeader    *hdi;     /* ϥե<head>Ǽ¤*/
   struct mssHeader    *hdo;     /* ϥե<head>Ǽ¤*/
+  struct mssHeader    *hdo2;    /* ϥե<head>Ǽ¤*/
   struct mssFPR       *fpr;     /* ϥե빽¤                */
   struct mssFPW       *fpw;     /* ϥե빽¤                */
+  struct mssFPW       *fpw2;    /* ϰϽϥե빽¤            */
   struct mssFldRecKey *frk=NULL;/* Хåե¤                */
   struct mssFields    *sf;      /* ȹܹ¤                  */
   int sorted;                   /*Ⱥѥå                 */
@@ -479,7 +503,7 @@ int main(int argc, char *argv[]){
   double *max;
   double *min;
   double rng;
-  int i,j,k;
+  int i,j,k,l;
 
 /*----------------------------------------------------------------------------*/
 /*                                                                      */
@@ -497,7 +521,7 @@ int main(int argc, char *argv[]){
   mssAddFieldsByFields(sf,optKEY.flds); /* -k ܤ򥽡ȹܤȤƥå    */
   mssSetFieldsSortPriority(sf);         /* ֹͥϿˤդ   */
   sorted=mssChkSorted(sf,hdi);          /* ȺѤå                 */
-    
+
 /*mssShowFields(sf);*/
 /*mssShowOption(opt);*/
 /*mssShowHeader(hdi);*/
@@ -507,7 +531,7 @@ int main(int argc, char *argv[]){
 /*----------------------------------------------------------------------------*/
   /*ϥإåν(ȥΥԡ)*/                              
   hdo=mssInitCpyHeader(hdi);
-    
+
   /*ϥإåܤɲ*/
   mssAddFieldsByFields(hdo->flds,hdi->flds);
 
@@ -522,7 +546,28 @@ int main(int argc, char *argv[]){
   /*ɸϥץ+إåν*/
   fpw=mssOpenFPW(optOTF.str,optZIP.set,0);                                    
   mssWriteHeader(hdo, fpw);
-    
+
+  /*ϰϽϥե륪ץ+إåν*/
+
+  /*ϥإåν(ȥΥԡ)*/
+  if(optRNG.set){
+    hdo2=mssInitCpyHeader(hdi);
+
+    for(l=0; l<optKEY.flds->cnt; l++){
+      mssAddFieldsByStr(hdo2->flds,MssFlds2name(optKEY.flds,l)); /*key*/
+    }
+    mssAddFieldsByStr(hdo2->flds,"fieldName"); /*fieldName*/
+    mssAddFieldsByStr(hdo2->flds,"bucketNo");  /*bucketNo*/
+    mssAddFieldsByStr(hdo2->flds,"rangeFrom"); /*rangeFrom*/
+    mssAddFieldsByStr(hdo2->flds,"rangeTo");   /*rangeTo*/
+
+    fpw2=mssOpenFPW(optRNG.str,optZIP.set,0);
+    mssWriteHeader(hdo2, fpw2);
+  }else{
+    hdo2=NULL;
+    fpw2=NULL;
+  }
+  
 /*----------------------------------------------------------------------------*/
 /*ᥤ롼                                                              */
 /*----------------------------------------------------------------------------*/
@@ -612,6 +657,32 @@ int main(int argc, char *argv[]){
       }
     }
 
+    /*ϰϥե*/
+    if(optRNG.set){
+      for(i=0; i<optFLD.flds->cnt; i++){
+        for(k=0; k<*(cutCnts+i); k++){
+          if(optREV.set)num=*(cutCnts+i)-k-1;
+          else          num=k;
+          mssSeekTopFRK(frk);
+          mssReadFldRecFRK(frk);
+          for(l=0; l<optKEY.flds->cnt; l++){
+            mssWriteStr(*(frk->pnt+MssFlds2num(optKEY.flds,l)),fpw2);
+            mssWriteDlm(fpw2);
+          }
+          mssWriteStr(MssFlds2name(optFLD.flds,i),fpw2);
+          mssWriteDlm(fpw2);
+          mssWriteInt(num+1,fpw2);
+          mssWriteDlm(fpw2);
+	  if(*(cutCnts+i)>0) mssWriteDbl((*(rTables+i)+k)->from,fpw2);
+	  else               mssWriteNull(fpw2);
+          mssWriteDlm(fpw2);
+          if(*(cutCnts+i)>0) mssWriteDbl((*(rTables+i)+k)->to  ,fpw2);
+	  else               mssWriteNull(fpw2);
+          mssWriteRet(fpw2);
+        }
+      }
+    }
+
     /**/
     mssSeekTopFRK(frk);
     while( EOF != mssReadFldRecFRK(frk) ){
@@ -666,11 +737,14 @@ int main(int argc, char *argv[]){
 /*----------------------------------------------------------------------------*/
 /*printf("allocCnt=%d\n",getAllocCnt());*/
   mssWriteFooter(fpw);    /* եåν             */
+  mssWriteFooter(fpw2);   /* եåν             */
   mssCloseFPR(fpr);       /* ϥեΥ     */
   mssCloseFPW(fpw);       /* ϥեΥ     */
+  mssCloseFPW(fpw2);      /* ϥեΥ     */
   mssFreeFields(sf);      /* ȹܹ¤Τΰ賫 */
   mssFreeHeader(hdi);     /* ϥإåΰ賫         */
   mssFreeHeader(hdo);     /* ϥإåΰ賫         */
+  mssFreeHeader(hdo2);    /* ϥإåΰ賫         */
   mssFreeOption(opt);     /* ץΰ賫         */
   mssShowEndMsg();        /* λå             */
   mssEnd(mssExitSuccess); /* λ                       */
