@@ -3,6 +3,8 @@
 /*----------------------------------------------------------------------------*/
 /* 1.0 : APIб 2003/06/20                                           */
 /* 1.1 : ͤǥȤĥǡưԶн(API) 2003/08/07     */
+/* 1.2 : -nȼäǡ̵եηǤΥ顼 2004/10/20          */
+/* 1.3 : եηǤΥ顼 2004/10/25                              */
 /*============================================================================*/
 
 #include <musashi.h>
@@ -12,7 +14,7 @@
 #include <xtjoinHelp.h>
 struct mssComHelp comHelp={
   "xtjoin",       /* ޥ̾       */
-  "1.1",          /* С       */
+  "1.3",          /* С       */
   HELPT,          /* ޥɥȥ */ 
   HELPS,          /*              */
   HELPE,          /*            */
@@ -565,10 +567,11 @@ int main(int argc, char *argv[]){
     while(1){
 
       /*Tra,MstξEndλ*/
-      if(frdT->eof==1 && statM==EOF) break;
+      if(statT==EOF && statM==EOF) break;
 
       /*TraEndλ*/
       if(frdT->eof==1){
+        if(statM==EOF) break;
         if(!optMNL.set) break;
       }
 
