@@ -3,6 +3,7 @@
 /*----------------------------------------------------------------------------*/
 /* 1.0 : APIб 2003/06/20                                           */
 /* 1.1 : ǥߥѲб 2003/09/26                                      */
+/* 1.2 : -eѥ᡼ɲ 2004/03/02                                          */
 /*============================================================================*/
    
 #include <musashi.h>
@@ -12,7 +13,7 @@
 #include <xtnumberHelp.h>
 struct mssComHelp comHelp={
   "xtnumber",     /* ޥ̾       */
-  "1.1",          /* С       */
+  "1.2",          /* С       */
   HELPT,          /* ޥɥȥ */
   HELPS,          /*              */
   HELPE,          /*            */
@@ -24,6 +25,20 @@ struct mssComHelp comHelp={
 
 extern struct mssGlobalVariables mssGV;
 
+int sameSRTfld(struct mssFldRecDbl *frd, MssOptFLD *optSRT, int kb){
+  int i;
+  int fldNo;
+
+  for(i=0; i<optSRT->flds->cnt; i++){
+    fldNo=MssFlds2num(optSRT->flds,i);
+    if(kb==1) return(0);
+    if(strcmp(*(frd->pnt[frd->new]+fldNo),*(frd->pnt[frd->old]+fldNo)) != 0){
+      return(0);
+    }
+  }
+  return(1);
+}
+
 int main(int argc, char *argv[]){
 /*============================================================================*/
 /* ץ                                                       */
@@ -106,6 +121,21 @@ int main(int argc, char *argv[]){
     INTC    /* ΥץΥ(Helpɽ)                         */
   };
 
+/*----------------------------------------------------------------------------*/
+/* ƱȹܤͤФ                                             */
+/*----------------------------------------------------------------------------*/
+  MssOptSEL optSAM={
+    OSEL,   /* ץ󥿥                                             */
+    "e",    /* (ʣʸԲ)                                   */
+    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
+    "seq",  /* ǥե(ʸˤ)                                       */
+    "seq,same,skip",
+            /* ǽͥꥹ                                           */
+    SAMT,   /* ΥץΥȥ(Helpɽ)                         */
+    SAMC,   /* ΥץΥ(Helpɽ)                         */
+    SAMS    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
+  };
+
 /*----------------------------------------------------------------------------*/
 /* Ϣ                                                                 */
 /*----------------------------------------------------------------------------*/
@@ -191,7 +221,7 @@ int main(int argc, char *argv[]){
 /*----------------------------------------------------------------------------*/
 /* ץޤȤ                                                       */
 /*----------------------------------------------------------------------------*/
-  void *opt[]={&optKEY,&optSRT,&optBGN,&optINT,&optBKY,&optFNM,&optSEQ,
+  void *opt[]={&optKEY,&optSRT,&optBGN,&optINT,&optSAM,&optBKY,&optFNM,&optSEQ,
                &optINF,&optOTF,&optZIP,&optTXT,&optTMP,NULL};
 
 /*============================================================================*/
@@ -205,7 +235,11 @@ int main(int argc, char *argv[]){
   struct mssFields    *sf;  /*ȹܹ¤                  */
   int sorted;               /*Ⱥѥå                */     
 
-  int count;             /*⤷ϥη*/
+  int seqCnt;             /*⤷ϥη*/
+  int samCnt;             /*⤷ϥη(Ʊ̹θ)*/
+  int same;
+  int skip;
+  int kb;
 
 /*----------------------------------------------------------------------------*/
 /*                                                                      */
@@ -229,6 +263,28 @@ int main(int argc, char *argv[]){
 /*mssShowOption(opt);*/
 /*mssShowHeader(hdi);*/
 
+  /*----------------------------*/
+  /*ѥ᡼֤ΰ¸å*/
+  /*----------------------------*/
+  /*-e same|skip  -BƱԲ*/
+  if( 0!=strcmp(optSAM.str,"seq") && optBKY.set ){
+    mssShowErrMsg("[-e same|skip] cannot be specified with -B");
+    exit(mssErrorNoDefault);
+  }
+
+  /*-e same|skip -sλ꤬ʤФʤʤ*/
+  if( 0!=strcmp(optSAM.str,"seq") && !optSRT.set ){
+    mssShowErrMsg("[-e same|skip] must be specified with -s");
+    exit(mssErrorNoDefault);
+  }
+
+  /*-s  -BƱԲ*/
+  if( optSRT.set && optBKY.set ){
+    mssShowErrMsg("-s cannot be specified with -B");
+    exit(mssErrorNoDefault);
+  }
+
+
 /*----------------------------------------------------------------------------*/
 /*ϥإåκȽ                                                    */
 /*----------------------------------------------------------------------------*/
@@ -255,12 +311,29 @@ int main(int argc, char *argv[]){
   SeqפǤʤ and sortedǤʤ -> presort + ֥졼
   ¾                         ->           ֥졼
 ------------------------------------------------------------------------------*/
+  same=0;
+  skip=0;
+  if( 0==strcmp(optSAM.str,"same") ){
+    same=1;
+    skip=0;
+  }
+
+  if( 0==strcmp(optSAM.str,"skip") ){
+    same=1;
+    skip=1;
+  }
+
   /*ȤɬѤʤХȤƥȺѤߥեȤƥץ*/
   if( !optSEQ.set && !sorted)
     fpr=mssReopenFPRsort(fpr,4,sf,hdi->flds->cnt,optTMP.str);
 
   frd=mssInitFRD(hdi->flds->cnt);
-  count=optBGN.val;
+  samCnt=optBGN.val-optINT.val;
+  seqCnt=optBGN.val-optINT.val;
+  if(optBKY.set){
+    seqCnt += optINT.val;
+  }
+  kb=1;
   while( EOF!=mssReadFRD(fpr,frd) ){
 
     /*֥쥤ν*/
@@ -268,20 +341,45 @@ int main(int argc, char *argv[]){
       if(frd->eof) break;
 
       if(optBKY.set){
-        count += optINT.val;
+        seqCnt += optINT.val;
       }else{
-        count= optBGN.val;
+        samCnt=optBGN.val-optINT.val;
+        seqCnt=optBGN.val-optINT.val;
       }
+      kb=1;
     }
 
     mssGV.inCnt++;
 
-    /*̾Ԥν(ͤη׻)*/
-    mssWriteFld(frd->pnt[frd->new],frd->fldCnt,MssFieldDelimStr,fpw);
-    mssWriteInt(count,fpw); mssWriteRet(fpw);
+    /*̾Ԥν(ȥå)*/
     if(!optBKY.set){
-      count += optINT.val;
+      if(same){
+        if( sameSRTfld(frd,&optSRT,kb) ){
+          seqCnt += optINT.val;
+        }else{
+          if(skip){
+            seqCnt += optINT.val;
+            samCnt = seqCnt;
+          }else{
+            seqCnt += optINT.val;
+            samCnt += optINT.val;
+          }
+        } 
+      /* -e seq ξ */
+      }else{
+        seqCnt += optINT.val;
+      }
     }
+
+    /*񤭽Ф*/
+    mssWriteFld(frd->pnt[frd->new],frd->fldCnt,MssFieldDelimStr,fpw);
+    if(same){
+      mssWriteInt(samCnt,fpw); mssWriteRet(fpw);
+    }else{
+      mssWriteInt(seqCnt,fpw); mssWriteRet(fpw);
+    }
+
+    kb=0;
     mssGV.outCnt++;
   }
   mssFreeFRD(frd);
