@@ -25,7 +25,7 @@ extern struct mssFields *fcat; /*ƥ*/
 extern struct mssFields *fcls; /*饹*/
 
 extern int ClassSize;          /*饹Υ*/
-extern iconv_t *icid;          /*Encoding*/
+//extern iconv_t *icid;          /*Encoding*/
 extern char mssXmlDomErrMsg[]; /*ɤ߹߻Υ顼å*/
 
 /**
@@ -193,9 +193,6 @@ struct Cost *readCost(char *fname,struct Class *cls){
   /*ե̾ꤵƤʤХǥեȥȤ֤*/
   if(fname==NULL) return(cost);
 
-  /*encoding*/
-  icid=iconv_open("EUC-JP","UTF-8");
-
   /*顼ؿ*/
   mssXmlDomErrMsg[0]='\0';
   xmlSetGenericErrorFunc(NULL,mssXmlDomErrHandler);
@@ -203,6 +200,9 @@ struct Cost *readCost(char *fname,struct Class *cls){
   /*ȥեparse*/
   doc = xmlParseFile(fname);
 
+  /*encoding*/
+  icid=iconv_open(doc->encoding,"UTF-8");
+
   if (doc == NULL ) {
     mssShowErrMsg(mssEncoding(mssXmlDomErrMsg,icid));
     exit(mssErrorNoDefault);
@@ -221,10 +221,11 @@ struct Cost *readCost(char *fname,struct Class *cls){
   /*롼ȥ̾(missClassificationCost)å*/
   if (xmlStrcmp(cur->name, (const xmlChar *) "missClassificationCost")) {
     xmlFreeDoc(doc);
-    mssShowErrMsg("root tag must be '<missClassificationCost>'");
+    mssShowErrMsg("cost file: root tag must be '<missClassificationCost>'");
     exit(mssErrorNoDefault);
   }
 
+
   /*<cost>Ǥɤ߹*/
   cur = cur->xmlChildrenNode;
   while (cur != NULL) {
@@ -232,10 +233,11 @@ struct Cost *readCost(char *fname,struct Class *cls){
       att1 = xmlGetProp(cur,"class");
       att2 = xmlGetProp(cur,"predict");
       att3 = xmlGetProp(cur,"value");
+
       clsNo=getClassNum(cls,mssEncoding(att1,icid));
       prdNo=getClassNum(cls,mssEncoding(att2,icid));
       if(clsNo<0 || prdNo<0){
-        mssShowErrMsg("invalid value on class or predict attribute");
+        mssShowErrMsg("cost file: invalid value on class or predict attribute");
         exit(mssErrorNoDefault);
       }
       cost->tbl[clsNo][prdNo] = atof(att3);
