@@ -5,6 +5,7 @@
 /* 0.2 : -cץɲ 2003/08/09                                          */
 /* 0.3 : -nץɲ,ѿ'('ǳ褦 2003/08/18                  */
 /* 0.4 : ݤ˴ؤХեå 2004/01/22                          */
+/* 0.5 : ꡼ 2004/10/31                                          */
 /*============================================================================*/
 
 #include <musashi.h>
@@ -14,7 +15,7 @@
 #include <xtformatHelp.h>
 struct mssComHelp comHelp={
   "xtformat",     /* ޥ̾       */
-  "0.4",          /* С       */
+  "0.5",          /* С       */
   HELPT,          /* ޥɥȥ */
   HELPS,          /*              */
   HELPE,          /*            */
@@ -47,7 +48,7 @@ enum DocType {
   ELSE
 };
 
-char keyStrings[1024]={'\0'}; /*ܤʸȤ¸*/
+char keyStrings[MssRecordMaxLen]={'\0'}; /*ܤʸȤ¸*/
 int keySum=0;
 
 void prnn(char *str,int len){
@@ -701,6 +702,7 @@ void setKeyTag(struct DocNode *doc, char *tag,struct PrefixNode *pt){
   }
   keySum+=keyCnt;
   doc->keyNo=keySum;
+  mssFree(keyStr); /* ɲ 2004/10/31 */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -807,6 +809,40 @@ void prnTxt(char *str){
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/* DocNode¤Τβ                                                        */
+/*----------------------------------------------------------------------------*/
+/* ɲ 2004/10/21 */
+void freeDocTree(struct DocNode *doc){
+  int i;
+
+  switch(doc->type){
+    case KEY:
+      break;
+    case LINE:
+      break;
+    case TXT:
+      break;
+    case IF:
+      mssCalFree(doc->cal);
+      break;
+    case ELIF:
+      mssCalFree(doc->cal);
+      break;
+    case THEN:
+      break;
+    case ELSE:
+      break;
+  }
+  if(doc->next!=NULL){
+    for(i=0; i<doc->nextCnt; i++){
+      freeDocTree(doc->next+i);
+    }
+  }
+  mssFree(doc->next);
+}
+
+
 void prnDocTree(struct DocNode *doc, int depth){
   int i;
 
@@ -975,6 +1011,7 @@ int main(int argc, char *argv[]){
 /* ץޤȤ                                                       */
 /*----------------------------------------------------------------------------*/
   void *opt[]={&optREP,&optINF,&optREF,&optNUL,&optOTF,&optZIP,NULL};
+  void *optInside[]={&optKEY,NULL}; /* ɲ 2004/10/31 */
 
   struct mssHeader *hdi; /*ϥե<head>Ǽ¤*/
   struct mssFPR    *fpr; /*ϥե빽¤*/
@@ -1100,13 +1137,16 @@ int main(int argc, char *argv[]){
   }
 
   /* optKey򤳤Ƕ*/
+  optInside[0]=&optKEY; optInside[1]=NULL; /* ɲ 2004/10/31 */
   if(keyStrings[0]=='\0'){
+    optKEY.type=OKEY;
     optKEY.set=0;
     optKEY.str=NULL;
     optKEY.cnt=0;
     optKEY.nam=NULL;
     optKEY.diffSame=1;
   }else{
+    optKEY.type=OKEY;
     optKEY.set=1;
     optKEY.str=mssStrdup(keyStrings);
     optKEY.cnt=0;
@@ -1199,6 +1239,9 @@ int main(int argc, char *argv[]){
     }
   }
 
+  freeDocTree(doc); /* ɲ 2004/10/31 */
+  mssFree(doc); /* ɲ 2004/10/31 */
+  mssFree(buf); /* ɲ 2004/10/31 */
   mssFreeFRD(frd);
   freePrefixTree(pt);
   mssFree(pt);
@@ -1207,9 +1250,12 @@ int main(int argc, char *argv[]){
 /*λ                                                                    */
 /*----------------------------------------------------------------------------*/
   mssCloseFPR(fpr);     /*ϥեΥ*/
+  mssFreeHeader(hdi);   /*ϥإåΰ賫          */
   mssCloseFPW(fpw);     /*ϥեΥ*/
   mssFreeOption(opt);   /*ץΰ賫*/
+  mssFreeOption(optInside); /*ץΰ賫*/
   mssShowEndMsg();      /* λå             */
+  mssEnd(mssExitSuccess); /* λ                       */
   return(0);            /* to avoid warning message   */
 }
 
