@@ -5,14 +5,14 @@
 /*============================================================================*/
 /*----------------------------------------------------------------------------*/
 /* R}h̃^Cg                                                         */
-/* R}hwv̍ŏ̍sɕ\\                                             */
+/* R}hwv̍ŏ̍sɕ\                                             */
 /*----------------------------------------------------------------------------*/
-#define HELPT "ڂ̑I"
+#define HELPT "R[h̕"
 
 /*----------------------------------------------------------------------------*/
 /* p[^                                                                 */
-/* R}hwv USAGE:Ƃ"L[[h XXXT"\\ ex)-k L[ږXg*/
-/* flag̃IvV̓R}hwvOPTIONS)Ƃĕ\\ ex)-z : Wo͂̈k*/
+/* R}hwv USAGE:Ƃ"L[[h XXXT"\ ex)-k L[ږXg*/
+/* flag̃IvV̓R}hwvOPTIONS)Ƃĕ\ ex)-z : Wo͂̈k*/
 /*----------------------------------------------------------------------------*/
 /*-f*/
 #define FLDT "ʍsږ"
@@ -37,8 +37,8 @@
 
 /*----------------------------------------------------------------------------*/
 /* v                                                                       */
-/* R}hwvSUMMARY:Ƃĕ\\                                     */
-/* HTMLwvmanɂ́AHELPD̏Ɍ`ŕ\\                   */
+/* R}hwvSUMMARY:Ƃĕ\                                     */
+/* HTMLwvmanɂ́AHELPD̏Ɍ`ŕ\                   */
 /*----------------------------------------------------------------------------*/
 #define HELPS "es̃f[^𕡎ʂAs̃f[^𐶐B"
 
@@ -56,20 +56,20 @@ xtduprec -f  -i dat.xt -o rsl.xt\n\
 
 /*----------------------------------------------------------------------------*/
 /* ҂ɂ                                                               */
-/* R}hwvɂ͕\\Ȃ                                             */
+/* R}hwvɂ͕\Ȃ                                             */
 /*----------------------------------------------------------------------------*/
 #define HELPA C_HELPA
 
 /*----------------------------------------------------------------------------*/
 /* oO|[g                                                               */
-/* o^΃R}hwvBUG REPORT:ɕ\\B                  */
-/* o^Ȃ(#define HELPB "")\\ȂB                           */
+/* o^΃R}hwvBUG REPORT:ɕ\B                  */
+/* o^Ȃ(#define HELPB "")\ȂB                           */
 /*----------------------------------------------------------------------------*/
 #define HELPB C_HELPB
 
 /*----------------------------------------------------------------------------*/
 /* z[y[W                                                               */
-/* o^΃R}hwvINFORMATION:ɕ\\B                 */
-/* o^Ȃ(#define HELPH "")\\ȂB                           */
+/* o^΃R}hwvINFORMATION:ɕ\B                 */
+/* o^Ȃ(#define HELPH "")\ȂB                           */
 /*----------------------------------------------------------------------------*/
 #define HELPH C_HELPH
