@@ -4,6 +4,10 @@
  * MUSASHIѤŪʴؿ
  * 20040129 mktimeؿΥޥʡbug fix
  *          tmpѥͭĶѿѹǤ褦
+ * 20040420 AllocInfϢؿľ(reallocбط)
+ * 20040428 mssPRand()ݥʬɲ
+ * 20040609 mssSort() ȴؿɲ
+ * 20041004 mssGV.argvargv[]mssStrdup褦ѹ
  * ============================================================================
  */
 
@@ -11,6 +15,7 @@
 #include <musashi/mssConfig.h>
 #include <musashi/mssOutput.h>
 #include <musashi/random.h>
+#include <musashi/qs5.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -136,6 +141,8 @@ void mssInit(int argc, char **argv, struct mssComHelp *comHelp)
 
   char *envStr;
   int sigDigits;
+  int i;
+  char buf[30];
 
   /*Хѿν*/
   mssGV.comHelp=comHelp;
@@ -147,10 +154,14 @@ void mssInit(int argc, char **argv, struct mssComHelp *comHelp)
   mssGV.inFldCnt=0;
   mssGV.quiet=0;
   mssGV.argc=argc;
-  mssGV.argv=argv;
+  mssGV.argv=mssMalloc(sizeof(char *)*argc,"mssInit");
+  for(i=0; i<argc; i++){
+    *(mssGV.argv+i)=mssStrdup(*(argv+i));
+  }
   mssGV.txtFlg=0;
   mssGV.allocDeb=0;
-  mssGV.allocInf=NULL;
+/*  mssGV.allocInf=NULL;*/
+  mssGV.allocFreeCnt=0;
   mssGV.allocCnt=0;
   mssGV.writeHeaderFlg=0;
   mssGV.writeFooterFlg=0;
@@ -168,7 +179,9 @@ void mssInit(int argc, char **argv, struct mssComHelp *comHelp)
   }else{
     sigDigits=MssSigDigits;
   }
-  sprintf(mssGV.sigDigitsForm,"%%.%dg",sigDigits);
+  /* sprintf(mssGV.sigDigitsForm,"%%.%dg",sigDigits); */
+  sprintf(buf,"%%.%dg",sigDigits);
+  mssGV.sigDigitsForm=mssStrdup(buf);
 
   /*ʥϿ*/
   mssSetSignalHandler();
@@ -180,15 +193,22 @@ void mssInit(int argc, char **argv, struct mssComHelp *comHelp)
  */
 void mssEnd(int status)
 {
+  int i;
+
   if(mssGV.usedTempFileFlg) delTmpFile();
   signal( SIGSEGV, SIG_IGN );
   signal( SIGPIPE, SIG_IGN );
   signal( SIGKILL, SIG_IGN );
   signal( SIGINT , SIG_IGN );
+
+  for(i=0; i<mssGV.argc; i++){
+    mssFree(*(mssGV.argv+i));
+  }
+  mssFree(mssGV.argv);
+  mssFree(mssGV.sigDigitsForm); /* ɲ 2004/10/31 */
   exit(status);
 }
 
-
 /**
  * # SECTION #
  * ----------------------------------------------------------------------------
@@ -214,54 +234,146 @@ void mssEnd(int status)
  * # FUNCTION #
  * ΰݡξ򥰥Хѿɲä
  */
-static void addAllocInf(int func, void *add, char *msg)
+static void addAllocInf(int func, void *add, int size,char *msg)
 {
-  mssGV.allocInf
-    =realloc(mssGV.allocInf,sizeof(struct mssAllocInf)*(mssGV.allocCnt+1));
-  (mssGV.allocInf+mssGV.allocCnt)->func=func;
-  (mssGV.allocInf+mssGV.allocCnt)->add =add;
-  (mssGV.allocInf+mssGV.allocCnt)->cnt =mssGV.allocFreeCnt;
-  (mssGV.allocInf+mssGV.allocCnt)->msg =strdup(msg);
-  (mssGV.allocInf+mssGV.allocCnt)->refNo=-1;
-  mssGV.allocCnt++;
-}
+  int i;
 
-/**
- * # FUNCTION #
- * ΰݡΰɽɽ
- * ɽγƹܤΰ̣Ϻ顢
- * 1: ֹ
- * 2: б륷ֹ
- * 3: ƤӽФ줿ؿ̾
- * 4: ΰΥɥ쥹(Freeξ0)
- * 5: (FreeξNULL)
- */
-void mssShowAllocInf(void)
-{
-  int i,j;
+  if(func==0) mssGV.allocFreeCnt--;
+  else        mssGV.allocFreeCnt++;
 
-  for(i=mssGV.allocCnt-1; i>0; i--){
-    if( (mssGV.allocInf+i)->func==0 && (mssGV.allocInf+i)->refNo==-1 ){
-      for(j=i-1; j>=0; j--){
-        if( (mssGV.allocInf+i)->add==(mssGV.allocInf+j)->add &&
-            (mssGV.allocInf+j)->refNo==-1 &&
-            (mssGV.allocInf+j)->func!=0   ){
-          (mssGV.allocInf+i)->refNo=j;
-          (mssGV.allocInf+j)->refNo=i;
+  mssGV.allocInf[mssGV.allocCnt].func=func;
+  mssGV.allocInf[mssGV.allocCnt].add =add;
+  mssGV.allocInf[mssGV.allocCnt].cnt =mssGV.allocFreeCnt;
+  mssGV.allocInf[mssGV.allocCnt].size =size;
+  mssGV.allocInf[mssGV.allocCnt].refNo=-1;
+  strcpy(mssGV.allocInf[mssGV.allocCnt].msg,msg);
+
+  if(func==0){
+    for(i=mssGV.allocCnt; i>=0; i--){
+      if( mssGV.allocInf[i].func!=0 && mssGV.allocInf[i].refNo==-1 ){
+        if( mssGV.allocInf[i].add==mssGV.allocInf[mssGV.allocCnt].add ){
+          mssGV.allocInf[i].refNo=mssGV.allocCnt;
+          mssGV.allocInf[mssGV.allocCnt].refNo=i;
           break;
         }
       }
     }
   }
 
+  mssGV.allocCnt++;
+}
+
+/**
+ * # FUNCTION #
+ * mssReallocǥΰ褬ݤ줿Υɥ쥹򸫤ƹ
+ */
+static void updAllocInf(int func, void *newAdd, void *oldAdd, int size, char *msg){
+  int i;
+  char buf[256];
+
+  if(newAdd==oldAdd){
+/*
+    sprintf(buf,"%s--same(%p,%p)",msg,newAdd,oldAdd);
+    addAllocInf(func, 0, 0, buf);
+*/
+    return;
+  }
+
+  if(oldAdd==NULL){
+    sprintf(buf,"%s--new",msg);
+    addAllocInf(func, newAdd, size, buf);
+    return;
+  }
+
+  for(i=mssGV.allocCnt-1; i>0; i--){
+    if( mssGV.allocInf[i].add==oldAdd && mssGV.allocInf[i].func!=0
+                                      && mssGV.allocInf[i].refNo==-1 ){
+      if(mssGV.allocInf[i].func!=func){
+        sprintf(buf,"%s--mismatched (%p,%p:%s)",mssGV.allocInf[i].msg,newAdd,oldAdd,msg);
+        strcpy(mssGV.allocInf[i].msg,buf);
+      }
+      mssGV.allocInf[i].func=func;
+      mssGV.allocInf[i].add =newAdd;
+      /*addAllocInf(func, 0, 0, msg);*/
+      return;
+    }
+  }
+  sprintf(buf,"%s--unmatched (%p,%p)",msg,newAdd,oldAdd);
+  addAllocInf(func, newAdd, size, buf);
+}
+
+/**
+ * # FUNCTION #
+ * ΰݡξ򥰥Хѿɲä
+ */
+void mssAddAllocLabel(char *msg)
+{
+  if(mssGV.allocDeb){
+    mssGV.allocInf[mssGV.allocCnt].func=6;
+    mssGV.allocInf[mssGV.allocCnt].add =NULL;
+    mssGV.allocInf[mssGV.allocCnt].cnt =0;
+    mssGV.allocInf[mssGV.allocCnt].size=0;
+    mssGV.allocInf[mssGV.allocCnt].refNo=0;
+    strcpy(mssGV.allocInf[mssGV.allocCnt].msg,msg);
+    mssGV.allocCnt++;
+  }
+}
+
+/**
+ * # FUNCTION #
+ * ΰݡΰɽɽ
+ */
+void mssShowAllocInf(void)
+{
+  int i;
+
+/*
+  for(i=mssGV.allocCnt-1; i>0; i--){
+    if( mssGV.allocInf[i].func==0 && mssGV.allocInf[i].refNo==-1 ){
+      for(j=i-1; j>=0; j--){
+        if( mssGV.allocInf[i].add==mssGV.allocInf[j].add &&
+            mssGV.allocInf[j].refNo==-1 &&
+            mssGV.allocInf[j].func!=0   ){
+          mssGV.allocInf[i].refNo=j;
+          mssGV.allocInf[j].refNo=i;
+          break;
+        }
+      }
+    }
+  }
+*/
+
+  fprintf(stderr,"========================================================\n");
+  fprintf(stderr,"ưŪΰγݡбɽ\n");
+  fprintf(stderr,"--------------------------------------------------------\n");
+  fprintf(stderr,"1: No. (or줿)\n");
+  fprintf(stderr,"2: ؿ\n");
+  fprintf(stderr,"      0:mssFree\n");
+  fprintf(stderr,"      1:mssMalloc\n");
+  fprintf(stderr,"      2:mssCalloc\n");
+  fprintf(stderr,"      3:mssRealloc\n");
+  fprintf(stderr,"      4:mssreallocLim\n");
+  fprintf(stderr,"      5:mssStrdup\n");
+  fprintf(stderr,"3: ()줿Ƭɥ쥹\n");
+  fprintf(stderr,"4: ݤ줿\n");
+  fprintf(stderr,"5: -бNo.\n");
+  fprintf(stderr,"6: λǤΥݿ(ǽԤ0ΤϤ)\n");
+  fprintf(stderr,"       (ǽԤ0Ǥʤв˺줬)\n");
+  fprintf(stderr,"7: mssFree,mssStrdupʳˤĤơƤӽФå\n");
+  fprintf(stderr,"========================================================\n");
   for(i=0; i<mssGV.allocCnt; i++){
-    fprintf(stderr,"%3d : %d %d %p %d %s\n",
-    i,
-    (mssGV.allocInf+i)->refNo,
-    (mssGV.allocInf+i)->func,
-    (mssGV.allocInf+i)->add ,
-    (mssGV.allocInf+i)->cnt ,
-    (mssGV.allocInf+i)->msg );
+    if( mssGV.allocInf[i].func==6){
+      fprintf(stderr,"%3d : Label : %s\n", i, mssGV.allocInf[i].msg );
+    }else{
+      fprintf(stderr,"%3d : %d %p %d %d %d %s\n",
+      i,
+      mssGV.allocInf[i].func,
+      mssGV.allocInf[i].add ,
+      mssGV.allocInf[i].size ,
+      mssGV.allocInf[i].refNo,
+      mssGV.allocInf[i].cnt ,
+      mssGV.allocInf[i].msg );
+    }
   }
 }
 
@@ -274,11 +386,11 @@ void mssFree(void *add)
 {
   if(add!=NULL){
     if(mssGV.allocDeb){
-      addAllocInf(0,add,"");
-      mssGV.allocFreeCnt--;
+      addAllocInf(0,add,0,"");
+/*      mssGV.allocFreeCnt--;*/
     }
     free(add);
-    //add=NULL;
+    /*add=NULL;*/
   }
 }
 
@@ -295,8 +407,8 @@ void *mssCalloc(int size, char *errMsg)
     mssEnd(mssErrorNoDefault);
   }
   if(mssGV.allocDeb) {
-    mssGV.allocFreeCnt++;
-    addAllocInf(1,p,errMsg);
+/*    mssGV.allocFreeCnt++;*/
+    addAllocInf(1,p,size,errMsg);
   }
   return(p);
 }
@@ -314,8 +426,8 @@ void *mssMalloc(int size, char *errMsg)
     mssEnd(mssErrorNoDefault);
   }
   if(mssGV.allocDeb){
-    mssGV.allocFreeCnt++;
-    addAllocInf(2,p,errMsg);
+/*    mssGV.allocFreeCnt++;*/
+    addAllocInf(2,p,size,errMsg);
   }
   return(p);
 }
@@ -332,9 +444,9 @@ void *mssRealloc(void *ptr, int size, char *errMsg)
     mssShowErrMsg("memory allocation error in [%s]",errMsg);
     mssEnd(mssErrorNoDefault);
   }
-  if(mssGV.allocDeb && ptr==NULL){
-    mssGV.allocFreeCnt++;
-    addAllocInf(3,p,errMsg);
+  if(mssGV.allocDeb){
+/*    mssGV.allocFreeCnt++;*/
+    updAllocInf(3,p,ptr,size,errMsg);
   }
   return(p);
 }
@@ -357,8 +469,8 @@ void *mssReallocLim(void *ptr, int size,int lim, char *errMsg)
     mssEnd(mssErrorNoDefault);
   }
   if(mssGV.allocDeb && ptr==NULL) {
-    mssGV.allocFreeCnt++;
-    addAllocInf(4,p,errMsg);
+/*    mssGV.allocFreeCnt++;*/
+    updAllocInf(4,p,ptr,size,errMsg);
   }
   return(p);
 }
@@ -373,8 +485,8 @@ char *mssStrdup(char *s)
   if(s==NULL) return(NULL);
   S=strdup(s);
   if(mssGV.allocDeb) {
-    mssGV.allocFreeCnt++;
-    addAllocInf(5,S,"");
+/*    mssGV.allocFreeCnt++;*/
+    addAllocInf(5,S,strlen(S),"");
   }
   return(S);
 }
@@ -643,7 +755,7 @@ char *mssItoA(int num)
  */
 char *mssFtoA(double num)
 {
-//  int i = 0;
+/*  int i = 0;*/
   char buf[2048];
 
   sprintf(buf,mssGV.sigDigitsForm,num);
@@ -719,6 +831,21 @@ double mssNRand(double m, double s)
 {
   return( gaussd(m,s) );
 }
+
+/**
+ * # FUNCTION #
+ * ݥʬ(ʿm)
+ */
+int mssPRand(double m)
+{
+  int k;
+  m=exp(m)*urnd0();
+  for(k=0; m>1.0;k++){
+    m*=urnd0();
+  }
+  return( k );
+}
+
 /**
  * # SECTION #
  * ----------------------------------------------------------------------------
@@ -805,6 +932,7 @@ char *mssSkipStrBlock(char *str, char startChr, char endChr)
  * ʤdup1ȤƤδؿѤƥȡʬ䤷硢
  * ˳ݤ줿ʸΰˤϡʸݥ󥿤κǽ
  * (ɥ쥹)Τ߳Ф褤
+ *  -> ѹ2004/10/28ƤǤʤФʤʤ
  * )
  *   char **s;
  *   int cnt;
@@ -818,13 +946,16 @@ char *mssSkipStrBlock(char *str, char startChr, char endChr)
  */
 char **mssTokByChr(char *string, char tok, int *cnt, int dup)
 {
+  char *orgStr;
   char *str;
   char *pos;
   char **list=NULL;
+  int i;
 
   *cnt=0;
   if(dup) str=mssStrdup(string);
   else    str=string;
+  orgStr=str;
 
   pos=str;
   while(1){
@@ -838,6 +969,15 @@ char **mssTokByChr(char *string, char tok, int *cnt, int dup)
     }
     str++;
   }
+
+  /* dupλ꤬listγǤstrdup ɲ 2004/10/28 */
+  if(dup){
+    for(i=0; i<*cnt; i++){
+      *(list+i)=mssStrdup(*(list+i));
+    }
+    mssFree(orgStr);
+  }
+
   return(list);
 }
 
@@ -853,6 +993,93 @@ int mssIsValidStr(char *str)
   return(1);
 }
 
+/**
+ * # FUNCTION #
+ * mssSortʸӥ롼
+ */
+static int cmpStrUp(const void **s1,const void **s2)
+{
+  return (strcmp(*(char **)s1,*(char **)s2));
+}
+
+/**
+ * # FUNCTION #
+ * mssSortʸ߽ӥ롼
+ */
+static int cmpStrDown(const void **s1,const void **s2)
+{
+  return (strcmp(*(char **)s1,*(char **)s2) * (-1) );
+}
+
+/**
+ * # FUNCTION #
+ * mssSortѿ;ӥ롼
+ */
+static int cmpNumUp(const void **s1,const void **s2)
+{
+  double n1,n2;
+
+  if(MssIsNull(*(char **)s1)) n1=-DBL_MAX;
+  else                        n1=atof(*s1);
+  if(MssIsNull(*(char **)s2)) n2=-DBL_MAX;
+  else                        n2=atof(*s2);
+  if(n1 > n2){
+    return(1);
+  }else if(n1 < n2){
+    return(-1);
+  }else{
+    return(0);
+  }
+}
+
+/**
+ * # FUNCTION #
+ * mssSortѿ͹߽ӥ롼
+ */
+static int cmpNumDown(const void **s1,const void **s2)
+{
+  double n1,n2;
+
+  if(MssIsNull(*(char **)s1)) n1=-DBL_MAX;
+  else                        n1=atof(*s1);
+  if(MssIsNull(*(char **)s2)) n2=-DBL_MAX;
+  else                        n2=atof(*s2);
+  if(n1 > n2){
+    return(-1);
+  }else if(n1 < n2){
+    return(1);
+  }else{
+    return(0);
+  }
+}
+
+/**
+ * # FUNCTION #
+ * ʸǡΥ(ͤȤƥȤ뤳Ȥǽ)
+ * flagǻꤹͤ
+ *     0:ʸ
+ *     1:ʸ߽
+ *     2:͹߽
+ *     3:;
+ */
+void mssSort(char **data, int cnt, int flag){
+  switch(flag){
+  case 0:
+    qsort5((char *)data, cnt, sizeof(char *), (int (*))cmpStrUp);
+    break;
+  case 1:
+    qsort5((char *)data, cnt, sizeof(char *), (int (*))cmpStrDown);
+    break;
+  case 2:
+    qsort5((char *)data, cnt, sizeof(char *), (int (*))cmpNumUp);
+    break;
+  case 3:
+    qsort5((char *)data, cnt, sizeof(char *), (int (*))cmpNumDown);
+    break;
+  }
+}
+
+
 /**
  * # SECTION #
  * ----------------------------------------------------------------------------
