@@ -3,6 +3,12 @@
  * ============================================================================
  * MUSASHIѤXMLtableǡϴϢδؿ
  * 20040123 mssReadFRKؿΥХ
+ * 20040519 -k -q ƱΥХ
+ * 20040803 ͽorս祽Ȥ򼨤ե饰CmpRevNummssSortDat¤Τ˰ܤ
+ *          㤨TraMstǡ줾̾諒ȡͥȤ򤷤Ȥȡ
+ *          ޤǥХǻäƤCmpRevNumǤнǤʤʤ뤿ᡣ
+ * 20040824 ե̾ˤgzopenfopenȤ櫓ͤѹ
+ * 20041031 ꡼
  * ============================================================================
  */
 
@@ -53,7 +59,8 @@ unsigned int ass_cnt;
 /*ΤǤϤʤɥ쥹ȤѤƤ롣*/
 static char readEnd[2]={0xff,0};
 
-
+/* qsort5ǰŪѤ sd->cmpRevNumåȤ */
+static struct mssSortDat *sdTmp;
 
 /*ե̾*/
 static char TFname[MssFileNameMaxLen];
@@ -102,35 +109,30 @@ extern struct mssGlobalVariables mssGV;
  *       [a]           [b]           [d]           [d]
  */
 
-static int CmpRevNum;    /*Ȥˤ륭Ӥ*/
-                         /* 0:ñƬʸӤǹԤ*/
-                         /* 1:͡սθӤԤ*/
-struct mssFields *SortFld; /*ȹܹ¤(initSDǥå)*/
-
 /**
  * # FUNCTION #
  * ӥ롼
- * CmpRevNumOnλs1,s2(char **)˥㥹ȤѤƤ롣
+ * sd->cmpRevNumOnλs1,s2(char **)˥㥹ȤѤƤ롣
  * s1ξ-1 s2ξ1֤ Ʊξ0֤
  * ȡʥȥĥ꡼ˤѤ롣
  */
-static int cmpKeyStr(char *s1, char *s2)
+static int cmpKeyStr(char *s1, char *s2, struct mssSortDat *sd)
 {
   int i;
   int no; /*ֹ*/
   int cmp; /*strcmpη*/
   double n1,n2;
 
-  if(!CmpRevNum){
+  if(!sd->cmpRevNum){
     /*̾ξstrcmpη̤餱ʤ*/
     /*s1<s2 -> -1 , s1>s2 -> 1, s1==s2 -> 0*/
     return(strcmp(s1,s2));
   }else{
     if(s1==readEnd) return(1);  /*s1λʤs2ξ*/
     if(s2==readEnd) return(-1); /*s2λʤs1ξ*/
-    for(i=0; i<SortFld->cnt; i++){
-      no=MssFlds2num(SortFld,i);
-      if(MssFlds2numFlg(SortFld,i)){
+    for(i=0; i<sd->flds->cnt; i++){
+      no=MssFlds2num(sd->flds,i);
+      if(MssFlds2numFlg(sd->flds,i)){
         if(MssIsNull(*((char **)s1+no))) n1=-DBL_MAX;
         else                         n1=atof(*((char **)s1+no));
         if(MssIsNull(*((char **)s2+no))) n2=-DBL_MAX;
@@ -138,19 +140,19 @@ static int cmpKeyStr(char *s1, char *s2)
 
         if(n1 > n2){
           /*礭λ n1>n2 -> n1ξ*/
-          if(MssFlds2revFlg(SortFld,i)) return(-1);
+          if(MssFlds2revFlg(sd->flds,i)) return(-1);
           /*;λ n1>n2 -> n2ξ*/
           else                          return(1);
         }else if(n1 < n2){
           /*礭λ n1<n2 -> n2ξ*/
-          if(MssFlds2revFlg(SortFld,i)) return(1);
+          if(MssFlds2revFlg(sd->flds,i)) return(1);
           /*;λ n1<n2 -> n1ξ*/
           else                           return(-1);
         }
       }else{
         /*ʸξstrcmpη̤餱ʤ*/
         if(0 != (cmp=strcmp( *((char **)s1+no),*((char **)s2+no))) ){
-          if(MssFlds2revFlg(SortFld,i)) return(cmp*(-1));
+          if(MssFlds2revFlg(sd->flds,i)) return(cmp*(-1));
           else                          return(cmp);
         }
       }
@@ -162,7 +164,7 @@ static int cmpKeyStr(char *s1, char *s2)
 /**
  * # FUNCTION #
  * ӥ롼(quickȴؿ)
- * CmpRevNumOnλs1,s2(char **)˥㥹ȤѤƤ롣
+ * sd->cmpRevNumOnλs1,s2(char **)˥㥹ȤѤƤ롣
  * s1ξ-1 s2ξ1֤ Ʊξ0֤
  */
 static int cmpKeyQst(const void **s1,const void **s2)
@@ -170,32 +172,32 @@ static int cmpKeyQst(const void **s1,const void **s2)
   register int i,no,cmp;
   double n1,n2;
 
-  if(!CmpRevNum){
+  if(!sdTmp->cmpRevNum){
     return (strcmp( *s1,*s2));
   }else{
-    for(i=0; i<SortFld->cnt; i++){
-      no=MssFlds2num(SortFld,i);
-      if(MssFlds2numFlg(SortFld,i)){
+    for(i=0; i<sdTmp->flds->cnt; i++){
+      no=MssFlds2num(sdTmp->flds,i);
+      if(MssFlds2numFlg(sdTmp->flds,i)){
         if(MssIsNull(*((char **)s1+no))) n1=-DBL_MAX;
         else                         n1=atof(*((char **)s1+no));
         if(MssIsNull(*((char **)s2+no))) n2=-DBL_MAX;
         else                         n2=atof(*((char **)s2+no));
         if(n1 > n2){
           /*礭λ n1>n2 -> n1ξ*/
-          if(MssFlds2revFlg(SortFld,i)) return(-1);
+          if(MssFlds2revFlg(sdTmp->flds,i)) return(-1);
 
           /*;λ n1>n2 -> n2ξ*/
           else                    return(1);
         }else if(n1 < n2){
           /*礭λ n1<n2 -> n2ξ*/
-          if(MssFlds2revFlg(SortFld,i)) return(1);
+          if(MssFlds2revFlg(sdTmp->flds,i)) return(1);
           /*;λ n1<n2 -> n1ξ*/
           else                    return(-1);
         }
       }else{
         /*ʸξstrcmpη̤餱ʤ*/
         if(0 != (cmp=strcmp( *((char **)s1+no),*((char **)s2+no))) ){
-          if(MssFlds2revFlg(SortFld,i)) return(cmp*(-1));
+          if(MssFlds2revFlg(sdTmp->flds,i)) return(cmp*(-1));
           else                    return(cmp);
         }
       }
@@ -303,14 +305,15 @@ static void TTsetLeaf(
 static void TTinsert(
   struct TTkey  *key, /*åȤʸʤɤξ*/
   struct TTnode *node,/*ȥåץΡ*/
-  int num){           /*ĥ꡼ΤǲܤΥΡ(꡼)˥åȤ뤫*/
+  int num,            /*ĥ꡼ΤǲܤΥΡ(꡼)˥åȤ뤫*/
+  struct mssSortDat *sd){
 
   TTsetLeaf(key,node,num);
 
   while(num>=2){
     /*Ԥ(ƥΡɤϿ)*/
     /*(-1) (0or1)*/
-    if( 0>cmpKeyStr((node+num)->key->str, (node+num)->brother->key->str) ){
+    if( 0>cmpKeyStr((node+num)->key->str, (node+num)->brother->key->str,sd) ){
       (node+num)->parent->key=(node+num)->key;
     }else{
       (node+num)->parent->key=(node+num)->brother->key;
@@ -327,12 +330,13 @@ static void TTinsert(
  */
 static void TTevaluate(
   struct TTnode *node,/*ȥåץΡ*/
-  int cnt){            /*դäѤο*/
+  int cnt,            /*դäѤο*/
+  struct mssSortDat *sd){
   int i;
 
   /*Ԥ(ƥΡɤϿ)*/
   for(i=cnt*2-1; i>=3; i=i-2){
-    if( 0>cmpKeyStr((node+i)->key->str, (node+i)->brother->key->str) ){
+    if( 0>cmpKeyStr((node+i)->key->str,(node+i)->brother->key->str,sd) ){
       (node+i)->parent->key=(node+i)->key;
     }else{
       (node+i-1)->parent->key=(node+i)->brother->key;
@@ -349,17 +353,18 @@ static void TTptree(
   struct TTnode *p,
   int num,   /*Ρֹ*/
   int h,     /*ڤο*/
-  int max) { /*Ρֹκ()*/
+  int max,   /*Ρֹκ()*/
+  struct mssSortDat *sd){
   int i;
 
   if(num<=max){
-    TTptree(p, num*2, h+1, max);
+    TTptree(p, num*2, h+1, max, sd);
 
     for(i=1; i<=h; i++) printf("   ");
     if( (p+num)->key->str == readEnd ){
       printf("%d NULL\n",(p+num)->num);
     }else{
-      if(CmpRevNum){
+      if(sd->cmpRevNum){
         printf("%d ", (p+num)->num);
         for(i=0; i<4; i++){
           printf("'%s'",*((char **)(p+num)->key->str+i));
@@ -371,7 +376,7 @@ static void TTptree(
       }
     }
 
-    TTptree(p, num*2+1, h+1, max);
+    TTptree(p, num*2+1, h+1, max, sd);
   }
 }
 
@@ -382,9 +387,10 @@ static void TTptree(
 void TTprintTree(
   char *s,
   struct TTnode *top,
-  int cnt){ /*դäѤο*/
+  int cnt,  /*դäѤο*/
+  struct mssSortDat *sd){
   printf("%s\n",s);
-  TTptree(top,1,0,cnt*2-1);
+  TTptree(top,1,0,cnt*2-1,sd);
   printf("------end\n");
 }
 
@@ -458,7 +464,7 @@ static void readTTkey(
   int bkt){
 
   /*ս硢ͥȤξ*/
-  if(CmpRevNum){
+  if(sd->cmpRevNum){
     if(EOF==mssReadFldRec(sd->iFile[bkt],sd->fr[bkt])){
       ttKey->str=readEnd;           /*EOFλʼ*/
       ttKey->bkt=bkt;               /*ХåֹΥå*/
@@ -555,9 +561,9 @@ static void setFirstLineTT(
   }
 
   /*ȡʥȥĥ꡼ǡͥԤ*/
-  TTevaluate(sd->tt,sd->bktCnt);
+  TTevaluate(sd->tt,sd->bktCnt,sd);
 
-  /*TTprintTree("############ before",sd->tt,sd->bktCnt);*/
+  /*TTprintTree("############ before",sd->tt,sd->bktCnt,sd);*/
 }
 
 /**
@@ -622,15 +628,15 @@ static void mergeTT( struct mssSortDat *sd )
         if(isReadEndTT(node->key->str)) break;   /*readEnd(ʼ)ʤ齪λ*/
         bkt=node->key->bkt;                      /*ФΥХåֹ*/
         /*Խ񤭽Ф*/
-        if(CmpRevNum){
+        if(sd->cmpRevNum){
           mssWriteFld((char **)node->key->str,sd->fldCnt,"\n",oFile);
         }else{
           mssWriteStr(node->key->str,oFile);
           mssWriteRet(oFile);
         }
         readTTkey(&ttKey, sd, bkt);              /*ιԤɤ߹*/
-        TTinsert(&ttKey,sd->tt,sd->bktCnt+bkt);  /*򥭥塼ɲ*/
-        /*TTprintTree("############ before",sd->tt,sd->bktCnt);*/
+        TTinsert(&ttKey,sd->tt,sd->bktCnt+bkt,sd); /*򥭥塼ɲ*/
+        /*TTprintTree("############ before",sd->tt,sd->bktCnt,sd);*/
       }
 
       TTfree(sd->tt);             /*塼γ*/
@@ -692,6 +698,7 @@ static void sort(
       qsort(wrkPnt,wrkCnt,sizeof(char *),
              (int (*)(const void *,const void *))cmpKeyQst);
 */
+      sdTmp=sd;
       qsort5((char *)wrkPnt,wrkCnt,sizeof(char *),
              (int (*))cmpKeyQst);
       writeBuf(wrkPnt,wrkCnt,getFname(sd->prefix,iCnt));
@@ -749,6 +756,7 @@ static void sort2(
     qsort(frm->pnt, frm->recCnt, sizeof(char *)*frm->fldCnt,
            (int (*)(const void *,const void *))cmpKeyQst);
 */
+    sdTmp=sd;
     qsort5((char *)frm->pnt, frm->recCnt, sizeof(char *)*frm->fldCnt,
              (int (*))cmpKeyQst);
 
@@ -782,23 +790,24 @@ static struct mssSortDat *initSD(struct mssFields *flds, int fldCnt, char *tmpDi
   int i;
   int    pid;              /*ץID(ե̾ΰ)*/
 
+  /* ΰ */
+  sd=mssCalloc(sizeof(struct mssSortDat),"initSD");
+
   /*ս硢ͥȻ꤬On(Global)*/
-  CmpRevNum=0;
+  sd->cmpRevNum=0;
   for(i=0; i<flds->cnt; i++){
     if( MssFlds2revFlg(flds,i) || MssFlds2numFlg(flds,i) ){
-      CmpRevNum=1;
+      sd->cmpRevNum=1;
     }
   }
 
   /*Хѿ˥å*/
-  SortFld=flds;
-
-  sd=mssCalloc(sizeof(struct mssSortDat),"initSD");
+  /* SortFld=flds; */
 
   sd->flds=flds; /*ȹܹ¤ΤΥå*/
 
   /*ս硢ͥȤξFldRec¤ΤȤ*/
-  if(CmpRevNum){
+  if(sd->cmpRevNum){
     for(i=0; i<PWayS; i++) sd->fr[i]=mssInitFldRec(fldCnt);
 
   /*ʸ󥽡ȤξRec¤ΤȤ*/
@@ -838,7 +847,7 @@ static void freeSD(struct mssSortDat *sd)
 
   TTfree(sd->tt); /*ȡʥȥĥ꡼Υ塼γ*/
 
-  if(CmpRevNum){
+  if(sd->cmpRevNum){
     for(i=0; i<PWayS; i++) mssFreeFldRec(sd->fr[i]);
   }else{
     for(i=0; i<PWayS; i++) mssFreeRec(sd->rec[i]);
@@ -876,7 +885,7 @@ static void preSort(
   struct mssFPR     *inFile){
 
 
-  if(CmpRevNum){/*ս硢ͥȻ꤬ initSDǥå*/
+  if(sd->cmpRevNum){/*ս硢ͥȻ꤬ initSDǥå*/
     sort2(sd,inFile);
   }else{
     setConv(sd);
@@ -983,14 +992,19 @@ static void fldCntErr(char **pnt, int fldCnt, int j, struct mssFPR *fp)
  * ܿۤʤХ顼λ
  * ιԤƬ֤֤
  */
+int hamCnt=0;
 static char *strToken(char **pnt, int fldCnt, int *chrCnt, struct mssFPR *fp)
 {
   char *str=fp->curPnt;
   int j;
+  int flg;
 
+  if(fp->sd==NULL)              flg=1;
+  else if(fp->sd->cmpRevNum==1) flg=1;
+  else                          flg=0;
 
   /*̾եξ⤷ϥȥեǿ͡ս祽Ȥξ*/
-  if(fp->sd==NULL || CmpRevNum){
+  if(flg){
     j=0; /*ܿ*/
     while(1){
       switch(*str){
@@ -1052,6 +1066,43 @@ static char *strToken(char **pnt, int fldCnt, int *chrCnt, struct mssFPR *fp)
   return(str);
 }
 
+/**
+ * # FUNCTION #
+ * 󥰥Хåեɽ(ǥХå)
+ */
+void showQue(struct mssFPR *fp,char *title){
+  int i,j,k;
+  char tmp;
+
+  fprintf(stderr,"-[QUE start %s]====================================\n",title);
+  fprintf(stderr,"enq=%d\n",fp->enq);
+  fprintf(stderr,"deq=%d\n",fp->deq);
+  fprintf(stderr,"full=%d\n",fp->full);
+
+  fprintf(stderr,"----------------------------------------\n");
+  for(i=0; i<MssRecordMaxLen; i++){
+   tmp=*(fp->buf+i);
+         if(tmp == '\0') fprintf(stderr,"@");
+    else if(tmp == '\n') fprintf(stderr,"$");
+    else                 fprintf(stderr,"%c",tmp);
+  }
+  fprintf(stderr,"\n");
+
+  for(i=0; i<fp->queCnt; i++){
+    fprintf(stderr,"[QUE %d]\n",i);
+    for(j=0; j<ReadCnt; j++){
+      fprintf(stderr,"[BUF %d] ",j);
+      for(k=0; k<PipeSize; k++){
+        tmp=*(fp->buf + i*fp->queSize + PipeSize*j+k+MssRecordMaxLen);
+             if(tmp == '\0') fprintf(stderr,"@");
+        else if(tmp == '\n') fprintf(stderr,"$");
+        else                 fprintf(stderr,"%c",tmp);
+      }
+      fprintf(stderr,"\n");
+    }
+  }
+}
+
 /**
  * # FUNCTION #
  * 󥰥Хåեؤϥǡɤ߹
@@ -1101,6 +1152,7 @@ static int readFPRfile(struct mssFPR *fp)
   /*塼full*/
   if(fp->full) return(-1);
 
+//showQue(fp,"BEFORE");
   /*̾եξ*/
   if(!fp->sort){
     /*enqƬʤХХåեǸMssRecordMaxLenʸХåեƬ˥ԡ*/
@@ -1119,11 +1171,6 @@ static int readFPRfile(struct mssFPR *fp)
                      sizeof(char) ,PipeSize, fp->fp);
       }
 
-      if( 0 > size ) {
-        mssShowErrMsg("file read error");
-        mssEnd(mssErrorNoDefault);
-      }
-
       /*eofޤɤ߹ߤξ硢eofΰΤ'\0'ɲ*/
       if(size<PipeSize){
         *(fp->buf+fp->enq*fp->queSize+PipeSize*i+MssRecordMaxLen+size) = '\0';
@@ -1142,6 +1189,7 @@ static int readFPRfile(struct mssFPR *fp)
   
     /*ХåեfullȽ*/
     if(fp->enq == fp->deq) fp->full=1;
+//showQue(fp,"AFTER");
 
   /*sortեξ*/
   /*̾եƱƯ򤹤*/
@@ -1176,7 +1224,7 @@ static int readFPRfile(struct mssFPR *fp)
       /*͡ս祽ȤξtnTree˹ܤڤäƤΤ᤹*/
       /*ν̵̤Τ褦˻פ뤬mssReadRec,mssReadFldRecʤɤ*/
       /*̤readؿ̾Ʊ˰Τǥƥʥ󥹤ڡ*/
-      if(CmpRevNum){
+      if(fp->sd->cmpRevNum){
         for(i=0; i<fp->sd->fldCnt-1; i++){
           fp->readPnt=strChrCpy(fp->readPnt,*((char **)node->key->str+i),
                                 MssFieldDelim);
@@ -1186,7 +1234,7 @@ static int readFPRfile(struct mssFPR *fp)
         fp->readPnt=strChrCpy(fp->readPnt,node->key->str,'\n');
       }
       readTTkey(&ttKey, fp->sd, bkt);                /*ιԤɤ߹*/
-      TTinsert(&ttKey,fp->sd->tt,fp->sd->bktCnt+bkt);/*塼ɲ*/
+      TTinsert(&ttKey,fp->sd->tt,fp->sd->bktCnt+bkt,fp->sd);/*塼ɲ*/
     }
     /*enqueueоݥХåե򣱤Ŀʤ*/
     fp->enq=NextQue(fp->enq,fp->queCnt);
@@ -1650,9 +1698,9 @@ void mssFreeFRK(struct mssFldRecKey *frk)
   if(0==access(frk->fName,R_OK & W_OK)) unlink(frk->fName);
   mssFree(frk->pnt);
   mssFree(frk->frPnt);
-  mssFree(frk);
   mssFreeFldRec(frk->fr);
   mssCloseFPR(frk->fpr);
+  mssFree(frk); /* ѹ 2004/10/31 */
 }
 
 /**
@@ -1731,7 +1779,11 @@ static void writeRecFRK(struct mssFldRecKey *frk,int recNo)
 int mssReadFRK(struct mssFPR *fp, struct mssFldRecKey *frk)
 {
   int i;
-
+  char *curTop;
+/*
+frkFlg=1;
+fprintf(stderr,"mssReadFRK start e,d=%d,%d\n",fp->enq,fp->deq);
+*/
   if(frk->eof) return(EOF);
 
   /**/
@@ -1745,7 +1797,8 @@ int mssReadFRK(struct mssFPR *fp, struct mssFldRecKey *frk)
  /* 2004/01/23 mssFreeFldRecθ˺ƤmssFreeFldRecƤӽФǽ뤿ᡢfrk->frNULLꥢ */
   frk->fr=NULL;
 
-  /*κǽ쥳ɤfrPntƬ˥ԡ롣*/
+  /* κǽ쥳ɤfrPntƬ˥ԡ롣*/
+  /* κǽ쥳ɤϺΥƱǤ롣 */
   if(!frk->firstRead){
     for(i=0; i<frk->fldCnt; i++){
       *(frk->frPnt+i)=*(frk->frPnt+frk->fldCnt*frk->recPnt+i);
@@ -1766,7 +1819,6 @@ int mssReadFRK(struct mssFPR *fp, struct mssFldRecKey *frk)
         if(!frk->byFile) {
           frk->fpw=mssOpenFPW(frk->fName,0,0);
         }
-
         /*frPntƤǽԤưեwrite*/
         for(i=0; i<frk->recPnt-1; i++){
           writeRecFRK(frk, i);
@@ -1784,6 +1836,8 @@ int mssReadFRK(struct mssFPR *fp, struct mssFldRecKey *frk)
       }
     }
 
+    curTop=fp->curPnt;
+
     /*eofθ*/
     if(isEOF(fp)){
       frk->eof=1;
@@ -1811,7 +1865,15 @@ int mssReadFRK(struct mssFPR *fp, struct mssFldRecKey *frk)
         frk->fr=mssInitFldRec(frk->fldCnt);
       }
 
-      fp->deq = PrevQue(fp->enq,fp->queCnt);
+      /* -k -q ΥХ 2004/05/19                        */
+      /* ֥졼deqϡenqΰȤϸ¤ʤ     */
+      /* ˤʤǽ⤢롣ǼΤ褦deq    */
+      /* strToken(ΥƬ)Ƭ(curTop) */
+      /* °QUEֹdeqȤ롣                               */
+      /* bufƬ+MssRecordMaxLenˤ0Ȥ롣 */
+      fp->deq = (curTop-fp->buf-MssRecordMaxLen)/fp->queSize;
+      if(fp->deq<0) fp->deq=0;
+/*      fp->deq = PrevQue(fp->enq,fp->queCnt);*/
       frk->keyNo++; /*ֹ̤򥫥ȥå*/
       frk->curRecNo=0;
       return(1);
@@ -1918,6 +1980,7 @@ int mssGetFldCntOnData(struct mssFPR *fp)
  */
 void mssSeekTopFPR(struct mssFPR *fp)
 {
+  int i;
 
   if(0==fp->fName && !fp->sort){
     mssShowErrMsg("can not seek on stdin");
@@ -1940,9 +2003,16 @@ void mssSeekTopFPR(struct mssFPR *fp)
 
   if(fp->sort){
     fp->readPnt=fp->curPnt;
+    /*塼β&ϥեΥ (ɲ 2004/10/31) */
+    TTfree(fp->sd->tt);
+    for(i=0; i<=fp->sd->iEnd-fp->sd->iStart; i++){
+      mssCloseFPR(fp->sd->iFile[i]);
+    }
+    
     setFirstLineTT(fp->sd, fp->sd->iStart,fp->sd->iEnd);
   }else{
-    gzseek(fp->zfd, 0, 0);
+    if(fp->zflg) gzseek(fp->zfd, 0, 0);
+    else          fseek(fp->fp , 0, 0);
     mssSkipToBody(fp);
   }
 }
@@ -1985,20 +2055,41 @@ static void fileOpenErr(char *fName)
  *         queCnt2ζҾǤʤФʤʤ,ex. 2,4,8,16...
  * queCnt4ꤹСǥեȤ16K*4=64KХȤΥǡХåե
  * ݤ뤳Ȥˤʤ롣
+ *
+ * ե̾γĥҤ.gzʤfzopen¾ξfopen(2004/08/24)
  */
 struct mssFPR *mssOpenFPR(char *fileName,int queCnt)
 {
   struct mssFPR *fp;
+  int len;
+  char *pos;
 
   fp=mssMalloc(sizeof(struct mssFPR), "openFPR");
 
   /*gzopen*/
-  if(fileName == NULL){
-    fp->zfd=gzdopen(0,"rb"); /*stdin*/
-  }else{
-    fp->zfd=gzopen(fileName,"rb");
+  fp->zflg=0;
+  if(fileName == NULL){  /*stdin open*/
+    fp->fp=stdin; //fopen("/dev/stdin","rb");
+    //if(fp->fp  == NULL) fileOpenErr(fileName);
+    fp->fName=mssStrdup("stdin");
+
+  }else{                 /* file open */
+    len=strlen(fileName);
+    if(len>=3){
+      pos=fileName+len-3;
+      if(*pos=='.' && *(pos+1)=='g' && *(pos+2)=='z'){
+        fp->zflg=1;
+      }
+    }
+    if(fp->zflg){
+      fp->zfd=gzopen(fileName,"rb");
+      if(fp->zfd == NULL) fileOpenErr(fileName);
+    }else{
+      fp->fp =fopen(fileName,"rb");
+      if(fp->fp  == NULL) fileOpenErr(fileName);
+    }
+    fp->fName=mssStrdup(fileName);
   }
-  if(fp->zfd == NULL) fileOpenErr(fileName);
 
   /*󥭥塼ǥ塼ݥ󥿤ν*/
   fp->sort=0;
@@ -2008,23 +2099,23 @@ struct mssFPR *mssOpenFPR(char *fileName,int queCnt)
   fp->last=0;
   fp->queCnt =queCnt;
   fp->queSize=ReadCnt*PipeSize;
+/*
   if(fileName==NULL){
-    fp->fName=mssMalloc(sizeof(char)*10,"openFPR");
-    strcpy(fp->fName,"stdin");
+    fp->fName=mssStrdup("stdin");
   }else{
-    fp->fName=mssMalloc(sizeof(char)*(strlen(fileName)+1),"openFPR");
-    strcpy(fp->fName,fileName);
+    fp->fName=mssStrdup(fileName);
   }
+*/
 
   /*Хåեΰγ*/
-  fp->buf=mssMalloc(sizeof(char)*fp->queSize*fp->queCnt+MssRecordMaxLen, "openFPR");
+  fp->buf=mssCalloc(sizeof(char)*(fp->queSize*fp->queCnt+MssRecordMaxLen+1), "openFPR");
 
   /*ȥݥ󥿤ХåեκǸ+1ˤäƤ(readFileΤ)*/
   fp->curPnt=fp->buf+fp->queSize*fp->queCnt+MssRecordMaxLen;
 
   fp->readPnt=NULL;
   fp->sd     =NULL;
-  fp->zflg   =1;
+  /*fp->zflg   =1;*/
 
   fp->recCnt=0;
   fp->chrCnt=0;
@@ -2110,7 +2201,14 @@ void mssCloseFPR(struct mssFPR *fp)
     freeSD(fp->sd);
     /*close(fp->fd);*/
   }else{
-    gzclose(fp->zfd);
+
+    if(fp->zflg){
+      gzclose(fp->zfd);
+    }else{ 
+      if(fp->fp != stdin){
+        fclose(fp->fp);
+      }
+    }
   }
   mssFree(fp->fName);
   mssFree(fp->buf);
@@ -2185,6 +2283,3 @@ void mssCloseFPU(struct mssFPR *fp)
   mssFree(fp->buf);
   mssFree(fp);
 }
-
-
-
