@@ -3,6 +3,9 @@
  * ============================================================================
  * MUSASHIѤXMLǡϴϢδؿ
  * iconvؿ(const char **)˥ץȥѴ 2003/12/24
+ * mssAddXmlTagAttribute* Ϣδؿ°̾NULLλʤˤ⤻˽λ
+ *   褦ѹ 2004/08/15
+ * ꡼ 2004/10/31
  * ============================================================================
  */
 
@@ -215,6 +218,74 @@ void mssFreeXmlTag(struct mssXmlTag *xmlTag)
   }
 }
 
+/**
+ * # FUNCTION #
+ * mssXmlTag¤Τ˻°̾(name)뤫ɤĴ١С
+ * °̾Ф(ʸ:value)򹹿롣ʤС°̾ͤɲä
+ * icidåȤƤ(NULLǤʤ)󥳡ǥ󥰤Ѵ򤪤ʤ
+ * xmlTag->attributesζǤ°̾򡢴Ǥͤݻ롣
+ * xmlTagNULLξϲ⤻˥꥿󤹤롣
+ */
+void mssAddXmlTagAttributeFmt(struct mssXmlTag *xmlTag, char *name, iconv_t *icid, char *fmt, ... )
+{
+  char *buf;
+  char *val;
+  char *nam;
+  char tmp[1024];
+  va_list args;
+
+  int i;
+
+  if(name==NULL || xmlTag==NULL){
+    return;
+  }
+
+  if(fmt==NULL){
+    mssShowErrMsg("internal error in addXmlAttribute");
+    exit(mssErrorNoDefault);
+  }
+
+  /*°̾*/
+  if(icid==NULL){
+    nam=mssStrdup(name);
+  }else{
+    nam=mssEncoding(name,icid);
+  }
+
+  /*°*/
+  va_start(args,fmt);
+  vsprintf(tmp,fmt,args);
+  va_end(args);
+
+  buf=mssMalloc(sizeof(char)*(strlen(tmp)+3),"updXmlTag");
+  strcpy(buf,"\"");
+  strcat(buf,tmp);
+  strcat(buf,"\"");
+  if(icid==NULL){
+    val=mssStrdup(buf);
+  }else{
+    val=mssEncoding(buf,icid);
+  }
+  mssFree(buf);
+ 
+  /*ϿƤ뤫Ĵ٤*/
+  for(i=0; i<xmlTag->attCnt; i=i+2){
+    /*פ°ä!!*/
+    if(strcmp(*(xmlTag->attributes+i),nam)==0){
+      mssFree(*(xmlTag->attributes+i+1));
+      *(xmlTag->attributes+i+1)=val;
+      return;
+    }
+  }
+  
+  /*value˰פ°̾ʤäΤɲ*/
+  xmlTag->attCnt+=2;
+  xmlTag->attributes=mssRealloc(xmlTag->attributes,
+                                sizeof(char *)*xmlTag->attCnt,"addXmlAtt");
+  *(xmlTag->attributes+xmlTag->attCnt-2)=nam;
+  *(xmlTag->attributes+xmlTag->attCnt-1)=val;
+}
+
 /**
  * # FUNCTION #
  * mssXmlTag¤Τ˻°̾(name)뤫ɤĴ١С
@@ -235,8 +306,7 @@ void mssAddXmlTagAttributeStr(struct mssXmlTag *xmlTag, char *name, char *value,
   }
 
   if(name==NULL || value==NULL){
-    mssShowErrMsg("internal error in addXmlAttribute");
-    exit(mssErrorNoDefault);
+    return;
   }
 
   /*°̾*/
@@ -291,15 +361,10 @@ void mssAddXmlTagAttributeInt(struct mssXmlTag *xmlTag, char *name, int value, i
   char *nam;
   int i;
 
-  if(xmlTag==NULL){
+  if(name==NULL || xmlTag==NULL){
     return;
   }
 
-  if(name==NULL){
-    mssShowErrMsg("internal error in addXmlAttribute");
-    exit(mssErrorNoDefault);
-  }
-
   /*°̾*/
   if(icid==NULL){
     nam=mssStrdup(name);
@@ -352,15 +417,10 @@ void mssAddXmlTagAttributeDbl(struct mssXmlTag *xmlTag, char *name, double value
   char *nam;
   int i;
 
-  if(xmlTag==NULL){
+  if(name==NULL || xmlTag==NULL){
     return;
   }
 
-  if(name==NULL){
-    mssShowErrMsg("internal error in addXmlAttribute");
-    exit(mssErrorNoDefault);
-  }
-
   /*°̾*/
   if(icid==NULL){
     nam=mssStrdup(name);
@@ -786,6 +846,7 @@ void mssWriteXmlTagInt(int indCnt, char *element, int content, int retFlg, iconv
   if(retFlg){
     mssWriteRet(fp);
   }
+  mssFree(buf); /* ɲ 2004/10/31 */
 }
 
 /**
@@ -816,6 +877,7 @@ void mssWriteXmlTagDbl(int indCnt, char *element, double content, int retFlg, ic
   if(retFlg){
     mssWriteRet(fp);
   }
+  mssFree(buf); /* ɲ 2004/10/31 */
 }
 
 /**
